#!/usr/bin/env python


# Resource Converter
#
# Converts a resource output generated by the DeRez tool and focuses on 
# 8img and 4img resource forks, outputting them to RAW image files
#
# Taylan Pince (taylanpince at gmail dot com)


import os
import sys

from binascii import a2b_hex
from optparse import OptionParser


def main():
    parser = OptionParser(usage="Usage: %prog [options] --file=FILE_PATH", version="%prog 0.1")

    parser.set_defaults(output_dir=os.getcwd())
    parser.add_option("-f", "--file", dest="file_path", help="Path to the resource file to parse")
    parser.add_option("-o", "--output-dir", dest="output_dir", help="Output path, by default the current working directory")

    (options, args) = parser.parse_args()
    
    if not options.file_path:
        parser.error("You have to specify a file path")

    file = open(options.file_path, "r")
    output = None

    for line in file.readlines():
        if output and "};" in line:
            output.close()
            output = None
        elif "8img" in line or "4img" in line:
            output = open(os.path.join(options.output_dir, "%s.raw" % line[line.find("(") + 1:line.find(")")]), "wb")
        elif output:
            output.write(a2b_hex(line[line.find('"') + 1:line.find('"', line.find('"') + 1)].replace(" ", "")))  

    file.close()


if __name__ == "__main__":
    main()
